<?php

namespace Drupal\random_reference_formatter\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Process the income data and returns a random entity.
 */
class RandomEntityController extends ControllerBase {

  /**
   * Constructor for random entity controller.
   */
  public function __construct() {
    $this->entityTypeManager();
  }

  /**
   * Get the random Entity.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The incoming request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response containing the content.
   */
  public function content(Request $request) {

    $candidatesGeneric = explode(',', $request->get('ids'));
    $viewMode = $request->get('view_mode');

    if (empty($candidatesGeneric)) {
      return $this->returnError();
    }

    $candidates = [];
    foreach ($candidatesGeneric as $idGeneric) {
      $id = strtok($idGeneric, '/');
      $entityType = strtok('');

      if (empty($idGeneric) || empty($entityType)) {
        continue;
      }

      $candidates[] = [
        'id' => $id,
        'entityType' => $entityType,
      ];
    }

    if (empty($viewMode)) {
      $viewMode = 'default';
    }

    try {

      // Get a random ID.
      $thatEntityData = array_values($candidates)[rand(0, count($candidates) - 1)];

      $entityStorage = $this->entityTypeManager
        ->getStorage($thatEntityData['entityType']);
      $entity = $entityStorage->load($thatEntityData['id']);

      if (!$entity) {
        return $this->returnError();
      }

      $entityViewBuilder = $this->entityTypeManager
        ->getViewBuilder($thatEntityData['entityType']);
      $entityView = $entityViewBuilder->view($entity, $viewMode);

      $randomEntity = \Drupal::service('renderer')
        ->render($entityView);

    }
    catch (\Exception$e) {
      return $this->returnError();
    }

    return new JsonResponse(['randomEntity' => $randomEntity]);
  }

  /**
   * Return error.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response containing error.
   */
  private function returnError() {
    return new JsonResponse(['error' => TRUE]);
  }

}
